// Copyright 2026 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package {{cookiecutter.java_package}};

import com.google.adk.web.AdkWebServer;
import org.springframework.boot.SpringApplication;

/**
 * Application entry point.
 * Starts the ADK web server with API, A2A, and Web UI support.
 */
public class Main {
  public static void main(String[] args) {
    System.setProperty("adk.agents.loader", "static");
    new SpringApplication(AdkWebServer.class, Agent.class).run(args);
  }
}
