// Copyright 2026 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Package agent contains the root agent implementation.
// This file is a placeholder and will be overridden by the specific agent template.
package agent

import (
	"context"

	"google.golang.org/adk/agent"
)

// NewRootAgent creates and returns the root agent.
// This is a placeholder implementation that will be overridden by the agent template.
func NewRootAgent(ctx context.Context) (agent.Agent, error) {
	panic("NewRootAgent not implemented - this file should be overridden by the agent template")
}
